@@ -151,8 +151,11 @@ static std::size_t compute_subwave_size(context& ctx, std::size_t n)
 /// {K}, then this will create a tensor of {K/N, N} where N is the number of
 /// split groups. To compute the number of split groups it finds the largest
 /// divisor that can divide K to make it less than min_size.
+/// The max_splits parameter limits the maximum number of split groups to prevent excessive
+/// splitting.
 static std::vector<shape> split_reduce(const std::vector<shape>& inputs,
-                                       std::size_t min_size = 1024)
+                                       std::size_t min_size   = 8192,
+                                       std::size_t max_splits = 16)
 {
     std::vector<shape> result;
     auto input_shape         = inputs.front();
@@ -175,7 +178,7 @@ static std::vector<shape> split_reduce(const std::vector<shape>& inputs,
     std::size_t n = 1;
     auto r        = input_shape.lens()[faxis];
     auto factors  = make_array(2, 3, 5, 7, 11);
-    while(r > min_size)
+    while(r > min_size and n < max_splits)
     {
         // NOLINTNEXTLINE(readability-qualified-auto)
         auto it = std::find_if(factors.begin(), factors.end(), [&](auto d) { return r % d == 0; });
