@@ -123,7 +123,7 @@ struct miopen_apply
         add_convolution_backwards_op();
         add_select_module_op();
         add_reshape_lazy_op();
-        add_group_query_attention_op();
+        add_concat_past_present_op();
         add_scan_slice_op();
     }
 
@@ -534,20 +534,9 @@ struct miopen_apply
         });
     }
 
-    void add_group_query_attention_op()
+    void add_concat_past_present_op()
     {
-        apply_map.emplace("gpu::gqa_rotary_embedding", [=](instruction_ref ins) {
-            auto s          = ins->get_shape();
-            auto output     = insert_allocation(ins, s);
-            auto new_inputs = ins->inputs();
-            new_inputs.push_back(output);
-            return mod->replace_instruction(
-                ins,
-                make_op("gpu::precompile_op", {{"op", to_value(ins->get_operator())}}),
-                new_inputs);
-        });
-
-        apply_map.emplace("gpu::concat_past_present", [=](instruction_ref ins) {
+        apply_map.emplace("concat_past_present", [=](instruction_ref ins) {
             return mod->replace_instruction(ins,
                                             make_op("gpu::precompile_op",
                                                     {{"op", to_value(ins->get_operator())},
