@@ -314,6 +314,11 @@ struct mlir_program
 
     MlirType make_type(shape::type_t t) const
     {
+        // non-computable type is not visit-able
+        if(t == shape::fp4x2_type)
+        {
+            return mlirFloat4E2M1FNTypeGet(ctx.get());
+        }
         MlirType result;
         shape::visit(t, [&](auto as) {
             if(as.type_enum() == shape::float_type)
@@ -363,8 +368,8 @@ struct mlir_program
     std::vector<MlirType> make_mlir_shapeds(const Range& r)
     {
         std::vector<MlirType> result;
-        std::transform(r.begin(), r.end(), std::back_inserter(result), [&](const auto& s) {
-            return make_mlir_shaped(s);
+        std::transform(r.begin(), r.end(), std::back_inserter(result), [&](const auto& i) {
+            return make_mlir_shaped(i);
         });
         return result;
     }
@@ -503,6 +508,17 @@ struct mlir_program
         {
         }
 
+        void set_operand_segment_sizes(const std::vector<int>& sizes)
+        {
+            MlirAttribute segment_sizes_attr =
+                mlirDenseI32ArrayGet(prog->ctx.get(), sizes.size(), sizes.data());
+            MlirNamedAttribute named_attr = mlirNamedAttributeGet(
+                mlirIdentifierGet(prog->ctx.get(),
+                                  mlirStringRefCreateFromCString("operandSegmentSizes")),
+                segment_sizes_attr);
+            mlirOperationStateAddAttributes(&op_state, 1, &named_attr);
+        }
+
         mlir_operation_state& add_attributes(const std::vector<named_attribute_t>& named_attrs)
         {
             auto attributes = prog->name_attributes(named_attrs);
@@ -602,22 +618,23 @@ struct mlir_program
     {
         auto names = m.get_parameter_names();
         std::sort(names.begin(), names.end());
-        std::vector<shape> inputs;
-        std::transform(names.begin(),
-                       names.end(),
-                       std::back_inserter(inputs),
-                       [&](const std::string& name) { return m.get_parameter_shape(name); });
+        std::vector<shape> input_shapes;
+        std::transform(
+            names.begin(),
+            names.end(),
+            std::back_inserter(input_shapes),
+            [&](const std::string& name) { return get_shape_for_mlir(m.get_parameter(name)); });
         std::vector<shape> outputs = m.get_output_shapes();
 
-        std::vector<MlirLocation> arg_locs(inputs.size(), location);
-        auto body_inputs   = make_mlir_shapeds(inputs);
+        std::vector<MlirLocation> arg_locs(input_shapes.size(), location);
+        auto body_inputs   = make_mlir_shapeds(input_shapes);
         mlir_region region = mlirRegionCreate();
         mlir_block fbody = mlirBlockCreate(body_inputs.size(), body_inputs.data(), arg_locs.data());
         MlirBlock result = fbody.get();
         mlirRegionAppendOwnedBlock(region.get(), fbody.release());
 
         auto ops = create_operation_state("func.func");
-        ops.add_attributes({{"function_type", make_function_type(inputs, outputs)},
+        ops.add_attributes({{"function_type", make_function_type(input_shapes, outputs)},
                             {"sym_name", sym_name},
                             {"kernel", std::string("mixr")},
                             {"arch", target_arch},
@@ -680,14 +697,54 @@ struct mlir_program
         return v;
     }
 
-    static shape get_shape(instruction_ref ins)
+    static bool input_is_unpack_fp4(instruction_ref ins)
     {
+        ins = instruction::get_output_alias(ins);
+        if(ins->name() == "reshape")
+        {
+            return input_is_unpack_fp4(ins->inputs().front());
+        }
+        if(ins->name() == "unpack_fp4")
+        {
+            return true;
+        }
+        return false;
+    }
+
+    static shape make_fp4_unpacked_shape(shape s)
+    {
+        auto new_lens    = s.lens();
+        auto new_strides = s.strides();
+        int fast_axis =
+            std::min_element(s.strides().cbegin(), s.strides().cend()) - s.strides().cbegin();
+        new_lens[fast_axis] *= 2;
+        for(auto i : range(new_strides.size()))
+        {
+            if(i != fast_axis)
+            {
+                new_strides.at(i) *= 2;
+            }
+        }
+        return {shape::fp4x2_type, new_lens, new_strides};
+    }
+
+    static shape get_shape_for_mlir(instruction_ref ins)
+    {
+        shape ret = ins->get_shape();
         if(ins->name() == "@return")
         {
             assert(ins->inputs().size() == 1);
-            return ins->inputs().front()->get_shape();
+            ret = ins->inputs().front()->get_shape();
         }
-        return ins->get_shape();
+        else if(input_is_unpack_fp4(ins))
+        {
+            ret = ret.with_type(shape::fp4x2_type);
+        }
+        else if(ins->get_shape().type() == shape::fp4x2_type)
+        {
+            ret = make_fp4_unpacked_shape(ret);
+        }
+        return ret;
     }
 
     static std::string get_symbol_name(const module& m)
@@ -716,7 +773,7 @@ struct mlir_program
         {
             if(ins->name() == "@param")
                 continue;
-            if(ins->name() == "contiguous")
+            if(contains({"contiguous", "unpack_fp4"}, ins->name()))
             {
                 ins_map[ins] = ins_map[ins->inputs().at(0)];
                 continue;
@@ -724,15 +781,16 @@ struct mlir_program
             auto name = get_name(ins);
             auto ops  = create_operation_state(name);
             ops.add_attribute_value(get_operator_value(ins));
+
+            // handles single output
             if(ins->name() != "@return")
-                ops.add_results({get_shape(ins)});
+                ops.add_results({get_shape_for_mlir(ins)});
 
             if(ins->name() == "@literal")
             {
                 literal r = ins->get_literal();
-                auto sh   = ins->get_shape();
 
-                MlirType shaped_type = make_mlir_shaped(sh);
+                MlirType shaped_type = make_mlir_shaped(ins->get_shape());
                 MlirType tensor_type = rocmlirMIXRShapedTypeAsTensor(shaped_type);
                 MlirAttribute mlir_value_attr =
                     mlirDenseElementsAttrRawBufferGet(tensor_type, r.get_shape().bytes(), r.data());
@@ -748,9 +806,25 @@ struct mlir_program
             std::vector<MlirValue> inputs;
             transform(
                 ins->inputs(), std::back_inserter(inputs), [&](auto i) { return ins_map.at(i); });
+            if(ins->name() == "quant_dot")
+            {
+                if(ins->inputs().size() == 4)
+                {
+                    // Specify operand segment sizes BEFORE creating the operation so MLIR sees it.
+                    // Use the canonical MLIR attribute name 'operandSegmentSizes'.
+                    const std::vector<int> seg_sizes = {1, 1, 1, 1};
+                    ops.set_operand_segment_sizes(seg_sizes);
+                }
+                else if(ins->inputs().size() == 2)
+                {
+                    const std::vector<int> seg_sizes = {1, 1, 0, 0};
+                    ops.set_operand_segment_sizes(seg_sizes);
+                }
+            }
             ops.add_operands(inputs);
 
             auto outputs = insert(fbody, std::move(ops));
+
             if(ins->name() != "@return")
             {
                 assert(outputs.size() == 1);
@@ -1201,6 +1275,7 @@ mlir_code_object compile_mlir(const context& migraphx_ctx,
         const std::lock_guard<std::mutex> lock(mutex);
         std::cout << mlir_print(&mlirOperationPrint, mod_op) << std::endl;
     }
+
     auto co = mp.compile(solution);
 
     co.expected_inputs = in_shapes;
@@ -1265,8 +1340,13 @@ tuning_config get_tuning_config_mlir(const context& migraphx_ctx,
     mlir_program mp;
     mp.set_gpu_properties(migraphx_ctx);
     mp.parse(m);
-    auto tc          = mp.get_tuning_config(exhaustive);
     const bool trace = enabled(MIGRAPHX_TRACE_MLIR{});
+    if(trace)
+    {
+        auto mod_op = mlirModuleGetOperation(mp.mmodule.get());
+        std::cout << mlir_print(&mlirOperationPrint, mod_op) << std::endl;
+    }
+    auto tc = mp.get_tuning_config(exhaustive);
     static std::mutex mutex;
     if(trace)
     {
